@@ -1,13 +1,21 @@
 package containerruntimeconfig
 
 import (
-	"encoding/json"
+	"context"
 	"fmt"
 	"reflect"
 	"time"
 
+	"github.com/clarketm/json"
 	igntypes "github.com/coreos/ignition/config/v2_2/types"
 	"github.com/golang/glog"
+	apicfgv1 "github.com/openshift/api/config/v1"
+	apioperatorsv1alpha1 "github.com/openshift/api/operator/v1alpha1"
+	configclientset "github.com/openshift/client-go/config/clientset/versioned"
+	cligoinformersv1 "github.com/openshift/client-go/config/informers/externalversions/config/v1"
+	cligolistersv1 "github.com/openshift/client-go/config/listers/config/v1"
+	operatorinformersv1alpha1 "github.com/openshift/client-go/operator/informers/externalversions/operator/v1alpha1"
+	operatorlistersv1alpha1 "github.com/openshift/client-go/operator/listers/operator/v1alpha1"
 	"github.com/vincent-petithory/dataurl"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/equality"
@@ -26,15 +34,7 @@ import (
 	"k8s.io/client-go/util/retry"
 	"k8s.io/client-go/util/workqueue"
 
-	apicfgv1 "github.com/openshift/api/config/v1"
-	apioperatorsv1alpha1 "github.com/openshift/api/operator/v1alpha1"
-	configclientset "github.com/openshift/client-go/config/clientset/versioned"
-	cligoinformersv1 "github.com/openshift/client-go/config/informers/externalversions/config/v1"
-	cligolistersv1 "github.com/openshift/client-go/config/listers/config/v1"
-	operatorinformersv1alpha1 "github.com/openshift/client-go/operator/informers/externalversions/operator/v1alpha1"
-	operatorlistersv1alpha1 "github.com/openshift/client-go/operator/listers/operator/v1alpha1"
 	mcfgv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
-	v1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 	ctrlcommon "github.com/openshift/machine-config-operator/pkg/controller/common"
 	mtmpl "github.com/openshift/machine-config-operator/pkg/controller/template"
 	mcfgclientset "github.com/openshift/machine-config-operator/pkg/generated/clientset/versioned"
@@ -275,7 +275,7 @@ func (ctrl *Controller) cascadeDelete(cfg *mcfgv1.ContainerRuntimeConfig) error
 		return nil
 	}
 	mcName := cfg.GetFinalizers()[0]
-	err := ctrl.client.MachineconfigurationV1().MachineConfigs().Delete(mcName, &metav1.DeleteOptions{})
+	err := ctrl.client.MachineconfigurationV1().MachineConfigs().Delete(context.TODO(), mcName, metav1.DeleteOptions{})
 	if err != nil && !errors.IsNotFound(err) {
 		return err
 	}
@@ -377,18 +377,18 @@ func (ctrl *Controller) handleImgErr(err error, key interface{}) {
 	ctrl.imgQueue.AddAfter(key, 1*time.Minute)
 }
 
-// generateOriginalContainerRuntimeConfigs returns rendered default storage, and crio config files
-func generateOriginalContainerRuntimeConfigs(templateDir string, cc *mcfgv1.ControllerConfig, role string) (*igntypes.File, *igntypes.File, *igntypes.File, *igntypes.File, error) {
+// generateOriginalContainerRuntimeConfigs returns rendered default storage, registries and policy config files
+func generateOriginalContainerRuntimeConfigs(templateDir string, cc *mcfgv1.ControllerConfig, role string) (*igntypes.File, *igntypes.File, *igntypes.File, error) {
 	// Render the default templates
 	rc := &mtmpl.RenderConfig{ControllerConfigSpec: &cc.Spec}
 	generatedConfigs, err := mtmpl.GenerateMachineConfigsForRole(rc, role, templateDir)
 	if err != nil {
-		return nil, nil, nil, nil, fmt.Errorf("generateMachineConfigsforRole failed with error %s", err)
+		return nil, nil, nil, fmt.Errorf("generateMachineConfigsforRole failed with error %s", err)
 	}
-	// Find generated storage.config, and crio.config
+	// Find generated storage.conf, registries.conf, and policy.json
 	var (
-		config, gmcStorageConfig, gmcCRIOConfig, gmcRegistriesConfig, gmcPolicyJSON *igntypes.File
-		errStorage, errCRIO, errRegistries, errPolicy                               error
+		config, gmcStorageConfig, gmcRegistriesConfig, gmcPolicyJSON *igntypes.File
+		errStorage, errRegistries, errPolicy                         error
 	)
 	// Find storage config
 	for _, gmc := range generatedConfigs {
@@ -398,14 +398,6 @@ func generateOriginalContainerRuntimeConfigs(templateDir string, cc *mcfgv1.Cont
 			break
 		}
 	}
-	// Find CRIO config
-	for _, gmc := range generatedConfigs {
-		config, errCRIO = findCRIOConfig(gmc)
-		if errCRIO == nil {
-			gmcCRIOConfig = config
-			break
-		}
-	}
 	// Find Registries config
 	for _, gmc := range generatedConfigs {
 		config, errRegistries = findRegistriesConfig(gmc)
@@ -422,11 +414,11 @@ func generateOriginalContainerRuntimeConfigs(templateDir string, cc *mcfgv1.Cont
 			break
 		}
 	}
-	if errStorage != nil || errCRIO != nil || errRegistries != nil || errPolicy != nil {
-		return nil, nil, nil, nil, fmt.Errorf("could not generate old container runtime configs: %v, %v, %v, %v", errStorage, errCRIO, errRegistries, errPolicy)
+	if errStorage != nil || errRegistries != nil || errPolicy != nil {
+		return nil, nil, nil, fmt.Errorf("could not generate old container runtime configs: %v, %v, %v", errStorage, errRegistries, errPolicy)
 	}
 
-	return gmcStorageConfig, gmcCRIOConfig, gmcRegistriesConfig, gmcPolicyJSON, nil
+	return gmcStorageConfig, gmcRegistriesConfig, gmcPolicyJSON, nil
 }
 
 func (ctrl *Controller) syncStatusOnly(cfg *mcfgv1.ContainerRuntimeConfig, err error, args ...interface{}) error {
@@ -449,8 +441,7 @@ func (ctrl *Controller) syncStatusOnly(cfg *mcfgv1.ContainerRuntimeConfig, err e
 		} else if newcfg.Status.Conditions[len(newcfg.Status.Conditions)-1].Message == newStatusCondition.Message {
 			newcfg.Status.Conditions[len(newcfg.Status.Conditions)-1] = newStatusCondition
 		}
-		_, updateErr := ctrl.client.MachineconfigurationV1().ContainerRuntimeConfigs().UpdateStatus(newcfg)
-
+		_, updateErr := ctrl.client.MachineconfigurationV1().ContainerRuntimeConfigs().UpdateStatus(context.TODO(), newcfg, metav1.UpdateOptions{})
 		return updateErr
 	})
 	// If an error occurred in updating the status just log it
@@ -523,7 +514,7 @@ func (ctrl *Controller) syncContainerRuntimeConfig(key string) error {
 		role := pool.Name
 		// Get MachineConfig
 		managedKey := getManagedKeyCtrCfg(pool)
-		mc, err := ctrl.client.MachineconfigurationV1().MachineConfigs().Get(managedKey, metav1.GetOptions{})
+		mc, err := ctrl.client.MachineconfigurationV1().MachineConfigs().Get(context.TODO(), managedKey, metav1.GetOptions{})
 		isNotFound := errors.IsNotFound(err)
 		if err != nil && !isNotFound {
 			return ctrl.syncStatusOnly(cfg, err, "could not find MachineConfig: %v", managedKey)
@@ -537,34 +528,42 @@ func (ctrl *Controller) syncContainerRuntimeConfig(key string) error {
 			}
 		}
 		// Generate the original ContainerRuntimeConfig
-		originalStorageIgn, originalCRIOIgn, _, _, err := generateOriginalContainerRuntimeConfigs(ctrl.templatesDir, controllerConfig, role)
+		originalStorageIgn, _, _, err := generateOriginalContainerRuntimeConfigs(ctrl.templatesDir, controllerConfig, role)
 		if err != nil {
 			return ctrl.syncStatusOnly(cfg, err, "could not generate origin ContainerRuntime Configs: %v", err)
 		}
 
-		var storageTOML, crioTOML []byte
+		var configFileList []generatedConfigFile
 		ctrcfg := cfg.Spec.ContainerRuntimeConfig
 		if ctrcfg.OverlaySize != (resource.Quantity{}) {
-			storageTOML, err = ctrl.mergeConfigChanges(originalStorageIgn, cfg, updateStorageConfig)
+			storageTOML, err := ctrl.mergeConfigChanges(originalStorageIgn, cfg, updateStorageConfig)
 			if err != nil {
 				glog.V(2).Infoln(cfg, err, "error merging user changes to storage.conf: %v", err)
+			} else {
+				configFileList = append(configFileList, generatedConfigFile{filePath: storageConfigPath, data: storageTOML})
 			}
 		}
-		if ctrcfg.LogLevel != "" || ctrcfg.PidsLimit != 0 || ctrcfg.LogSizeMax != (resource.Quantity{}) {
-			crioTOML, err = ctrl.mergeConfigChanges(originalCRIOIgn, cfg, updateCRIOConfig)
-			if err != nil {
-				glog.V(2).Infoln(cfg, err, "error merging user changes to crio.conf: %v", err)
-			}
-		}
-		if isNotFound {
-			tempIgnCfg := ctrlcommon.NewIgnConfig()
-			mc = mtmpl.MachineConfigFromIgnConfig(role, managedKey, &tempIgnCfg)
-		}
 
-		mc.Spec.Config = createNewIgnition([]ignitionConfig{
-			{filePath: storageConfigPath, data: storageTOML},
-			{filePath: crioConfigPath, data: crioTOML},
-		})
+		// Create the cri-o drop-in files
+		if ctrcfg.LogLevel != "" || ctrcfg.PidsLimit != 0 || ctrcfg.LogSizeMax != (resource.Quantity{}) {
+			crioFileConfigs := createCRIODropinFiles(cfg)
+			configFileList = append(configFileList, crioFileConfigs...)
+		}
+
+		if isNotFound {
+			tempIgnCfg := ctrlcommon.NewIgnConfig()
+			mc, err = mtmpl.MachineConfigFromIgnConfig(role, managedKey, tempIgnCfg)
+			if err != nil {
+				return ctrl.syncStatusOnly(cfg, err, "could not create MachineConfig from new Ignition config: %v", err)
+			}
+		}
+
+		ctrRuntimeConfigIgn := createNewIgnition(configFileList)
+		rawCtrRuntimeConfigIgn, err := json.Marshal(ctrRuntimeConfigIgn)
+		if err != nil {
+			return ctrl.syncStatusOnly(cfg, err, "error marshalling container runtime config Ignition: %v", err)
+		}
+		mc.Spec.Config.Raw = rawCtrRuntimeConfigIgn
 
 		mc.SetAnnotations(map[string]string{
 			ctrlcommon.GeneratedByControllerVersionAnnotationKey: version.Hash,
@@ -576,9 +575,9 @@ func (ctrl *Controller) syncContainerRuntimeConfig(key string) error {
 		if err := retry.RetryOnConflict(updateBackoff, func() error {
 			var err error
 			if isNotFound {
-				_, err = ctrl.client.MachineconfigurationV1().MachineConfigs().Create(mc)
+				_, err = ctrl.client.MachineconfigurationV1().MachineConfigs().Create(context.TODO(), mc, metav1.CreateOptions{})
 			} else {
-				_, err = ctrl.client.MachineconfigurationV1().MachineConfigs().Update(mc)
+				_, err = ctrl.client.MachineconfigurationV1().MachineConfigs().Update(context.TODO(), mc, metav1.UpdateOptions{})
 			}
 			return err
 		}); err != nil {
@@ -681,12 +680,16 @@ func (ctrl *Controller) syncImageConfig(key string) error {
 			if err != nil {
 				return err
 			}
-			mc, err := ctrl.client.MachineconfigurationV1().MachineConfigs().Get(managedKey, metav1.GetOptions{})
+			rawRegistriesIgn, err := json.Marshal(registriesIgn)
+			if err != nil {
+				return fmt.Errorf("could not encode registries Ignition config: %v", err)
+			}
+			mc, err := ctrl.client.MachineconfigurationV1().MachineConfigs().Get(context.TODO(), managedKey, metav1.GetOptions{})
 			if err != nil && !errors.IsNotFound(err) {
 				return fmt.Errorf("could not find MachineConfig: %v", err)
 			}
 			isNotFound := errors.IsNotFound(err)
-			if !isNotFound && equality.Semantic.DeepEqual(registriesIgn, mc.Spec.Config) {
+			if !isNotFound && equality.Semantic.DeepEqual(rawRegistriesIgn, mc.Spec.Config.Raw) {
 				// if the configuration for the registries is equal, we still need to compare
 				// the generated controller version because during an upgrade we need a new one
 				mcCtrlVersion := mc.Annotations[ctrlcommon.GeneratedByControllerVersionAnnotationKey]
@@ -697,9 +700,12 @@ func (ctrl *Controller) syncImageConfig(key string) error {
 			}
 			if isNotFound {
 				tempIgnCfg := ctrlcommon.NewIgnConfig()
-				mc = mtmpl.MachineConfigFromIgnConfig(role, managedKey, &tempIgnCfg)
+				mc, err = mtmpl.MachineConfigFromIgnConfig(role, managedKey, tempIgnCfg)
+				if err != nil {
+					return fmt.Errorf("could not create MachineConfig from new Ignition config: %v", err)
+				}
 			}
-			mc.Spec.Config = *registriesIgn
+			mc.Spec.Config.Raw = rawRegistriesIgn
 			mc.ObjectMeta.Annotations = map[string]string{
 				ctrlcommon.GeneratedByControllerVersionAnnotationKey: version.Hash,
 			}
@@ -713,9 +719,9 @@ func (ctrl *Controller) syncImageConfig(key string) error {
 			}
 			// Create or Update, on conflict retry
 			if isNotFound {
-				_, err = ctrl.client.MachineconfigurationV1().MachineConfigs().Create(mc)
+				_, err = ctrl.client.MachineconfigurationV1().MachineConfigs().Create(context.TODO(), mc, metav1.CreateOptions{})
 			} else {
-				_, err = ctrl.client.MachineconfigurationV1().MachineConfigs().Update(mc)
+				_, err = ctrl.client.MachineconfigurationV1().MachineConfigs().Update(context.TODO(), mc, metav1.UpdateOptions{})
 			}
 
 			return err
@@ -739,7 +745,7 @@ func registriesConfigIgnition(templateDir string, controllerConfig *mcfgv1.Contr
 	)
 
 	// Generate the original registries config
-	_, _, originalRegistriesIgn, originalPolicyIgn, err := generateOriginalContainerRuntimeConfigs(templateDir, controllerConfig, role)
+	_, originalRegistriesIgn, originalPolicyIgn, err := generateOriginalContainerRuntimeConfigs(templateDir, controllerConfig, role)
 	if err != nil {
 		return nil, fmt.Errorf("could not generate origin ContainerRuntime Configs: %v", err)
 	}
@@ -764,7 +770,7 @@ func registriesConfigIgnition(templateDir string, controllerConfig *mcfgv1.Contr
 			return nil, fmt.Errorf("could not update policy json with new changes: %v", err)
 		}
 	}
-	registriesIgn := createNewIgnition([]ignitionConfig{
+	registriesIgn := createNewIgnition([]generatedConfigFile{
 		{filePath: registriesConfigPath, data: registriesTOML},
 		{filePath: policyConfigPath, data: policyJSON},
 	})
@@ -773,13 +779,13 @@ func registriesConfigIgnition(templateDir string, controllerConfig *mcfgv1.Contr
 
 // RunImageBootstrap generates MachineConfig objects for mcpPools that would have been generated by syncImageConfig,
 // except that mcfgv1.Image is not available.
-func RunImageBootstrap(templateDir string, controllerConfig *v1.ControllerConfig, mcpPools []*v1.MachineConfigPool, icspRules []*apioperatorsv1alpha1.ImageContentSourcePolicy) ([]*v1.MachineConfig, error) {
+func RunImageBootstrap(templateDir string, controllerConfig *mcfgv1.ControllerConfig, mcpPools []*mcfgv1.MachineConfigPool, icspRules []*apioperatorsv1alpha1.ImageContentSourcePolicy) ([]*mcfgv1.MachineConfig, error) {
 	// ImageConfig is not available.
 	insecureRegs := []string(nil)
 	blockedRegs := []string(nil)
 	allowedRegs := []string(nil)
 
-	var res []*v1.MachineConfig
+	var res []*mcfgv1.MachineConfig
 	for _, pool := range mcpPools {
 		role := pool.Name
 		managedKey := getManagedKeyReg(pool)
@@ -789,10 +795,10 @@ func RunImageBootstrap(templateDir string, controllerConfig *v1.ControllerConfig
 		if err != nil {
 			return nil, err
 		}
-
-		tempIgnCfg := ctrlcommon.NewIgnConfig()
-		mc := mtmpl.MachineConfigFromIgnConfig(role, managedKey, &tempIgnCfg)
-		mc.Spec.Config = *registriesIgn
+		mc, err := mtmpl.MachineConfigFromIgnConfig(role, managedKey, registriesIgn)
+		if err != nil {
+			return nil, err
+		}
 		// Explicitly do NOT set GeneratedByControllerVersionAnnotationKey so that the first run of the non-bootstrap controller
 		// always rebuilds registries.conf (with the insecureRegs/blockedRegs values actually available).
 		mc.ObjectMeta.OwnerReferences = []metav1.OwnerReference{
@@ -839,7 +845,7 @@ func (ctrl *Controller) popFinalizerFromContainerRuntimeConfig(ctrCfg *mcfgv1.Co
 }
 
 func (ctrl *Controller) patchContainerRuntimeConfigs(name string, patch []byte) error {
-	_, err := ctrl.client.MachineconfigurationV1().ContainerRuntimeConfigs().Patch(name, types.MergePatchType, patch)
+	_, err := ctrl.client.MachineconfigurationV1().ContainerRuntimeConfigs().Patch(context.TODO(), name, types.MergePatchType, patch, metav1.PatchOptions{})
 	return err
 }
 
